# Generated by Django 2.0.2 on 2018-03-19 11:43

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='RidiUser',
            fields=[
                ('password', models.CharField(max_length=128, verbose_name='password')),
                ('last_login', models.DateTimeField(blank=True, null=True, verbose_name='last login')),
                ('u_idx', models.AutoField(editable=False, primary_key=True, serialize=False, verbose_name='u_idx')),
            ],
            options={
                'verbose_name': '사용자 계정',
                'verbose_name_plural': '사용자 계정 리스트',
                'db_table': 'ridi_user',
            },
        ),
    ]
